@@ -31,6 +31,7 @@ import Data.Char (isDigit, isAscii, isLetter)
 import qualified Data.Map as M
 import qualified Data.Text as T
 import Data.Text (Text)
+import Data.Ratio ((%))
 import Data.Maybe (catMaybes, fromJust, mapMaybe)
 import Text.Parsec hiding (label)
 import Text.Parsec.Error
@@ -52,7 +53,27 @@ type TP = Parser
 -- The parser
 
 expr1 :: TP Exp
-expr1 = choice
+expr1 = do
+  e <- expr2
+  -- check for primes and add them as a subscript
+  -- in TeX ' is shorthand for ^{\prime}
+  primes <- many (char '\'')
+  let getPrimes cs = case cs of
+                       "" -> ""
+                       "'" -> "\x2032"
+                       "''" -> "\x2033"
+                       "'''" -> "\x2034"
+                       "''''" -> "\x2057"
+                       _ -> "\x2057" ++ getPrimes (drop 4 cs)
+  ignorable
+  case getPrimes primes of
+    "" -> return e
+    cs -> return $ case e of
+                     ESub b sub -> ESubsup b sub (ESymbol Pun (T.pack cs))
+                     _ -> ESuper e (ESymbol Pun (T.pack cs))
+
+expr2 :: TP Exp
+expr2 = choice
           [ inbraces
           , variable
           , number
@@ -62,7 +83,7 @@ expr1 = choice
           , enclosure
           , hyperref
           , command
-          ] <* ignorable
+          ]
 
 -- | Parse a formula, returning a list of 'Exp'.
 readTeX :: Text -> Either Text [Exp]
@@ -202,6 +223,7 @@ command = try $ do
   choice
     [ text c
     , styled c
+    , colored c
     , root c
     , xspace c
     , mathop c
@@ -232,7 +254,7 @@ operatorname = do
     ctrlseq "operatorname"
     -- these are slightly different but we won't worry about that here...
     convertible <- (char '*' >> spaces >> return True) <|> return False
-    tok' <- texToken
+    tok' <- texSymbol <|> braces (manyExp expr2) <|> texChar
     tok'' <- (EMathOperator <$> (expToOperatorName tok'))
            <|> return (EStyled TextNormal [tok'])
     return (tok'', convertible)
@@ -337,7 +359,7 @@ many1Exp :: TP Exp -> TP Exp
 many1Exp = manyExp' True
 
 inbraces :: TP Exp
-inbraces = braces (manyExp expr)
+inbraces = braces (ignorable *> manyExp expr)
 
 texToken :: TP Exp
 texToken = texSymbol <|> inbraces <|> texChar
@@ -407,19 +429,13 @@ delimited = do
 
 delimitedImplicit :: TP Exp
 delimitedImplicit = try $ do
-  openc <- lexeme $ oneOf "()[]|"
-  closec <- case openc of
-                 '(' -> return ')'
-                 '[' -> return ']'
-                 '|' -> return '|'
-                 _   -> mzero
-  let closer = lexeme $ char closec
+  (opent, closer) <- (("\x2016", "\x2016" <$ ctrlseq "rVert") <$ ctrlseq "lVert")
   contents <- concat <$>
               many (try $ ((:[]) . Left  <$> middle)
                       <|> (map Right . unGrouped <$>
                              many1Exp (notFollowedBy closer *> expr)))
-  _ <- closer
-  return $ EDelimited (T.singleton openc) (T.singleton closec) contents
+  closet <- T.pack <$> closer
+  return $ EDelimited opent closet contents
 
 scaled :: Text -> TP Exp
 scaled cmd = do
@@ -444,18 +460,21 @@ endLineAMS = lexeme $ try $ do
 
 arrayLine :: TP ArrayLine
 arrayLine =
-  sepBy1 (unGrouped <$>
-    manyExp (try (ignorable' *> notFollowedBy (('\n' <$ ctrlseq "end") <|> endLine)) *>
-               expr <*
-               ignorable')) (symbol "&")
-  where ignorable' = ignorable >>
-                     optional (try (ctrlseq "hline" >> ignorable'))
+  sepBy1
+    (ignorable' *>
+      (unGrouped <$>
+        manyExp (try (notFollowedBy (('\n' <$ ctrlseq "end") <|> endLine)) *>
+                   expr <* ignorable'))) (symbol "&")
+  where ignorable' = ignorable >> optional (try (ctrlseq "hline" >> ignorable'))
   -- we don't represent the line, but it shouldn't crash parsing
 
 arrayAlignments :: TP [Alignment]
 arrayAlignments = mconcat <$>
-  braces (many (((:[]) . letterToAlignment <$> letter)
+  braces (many (
+                ((:[]) . letterToAlignment <$> letter)
             <|> ([] <$ char '|')
+            <|> ([] <$ oneOf " \t")
+            <|> ([] <$ ((char '@' <|> char '!') <* inbraces))
             <|> (do char '*'
                     num <- T.pack <$> braces (many1 digit)
                     cols <- arrayAlignments
@@ -676,6 +695,13 @@ styled c = do
                        _           -> f [x]
        Nothing  -> mzero
 
+colored :: Text -> TP Exp
+colored "\\color" = do
+  _ <- inbraces -- skip the color
+  -- in the future we might add color to the types or to the styles
+  texSymbol <|> inbraces <|> texChar
+colored _ = mzero
+
 -- note: sqrt can be unary, \sqrt{2}, or binary, \sqrt[3]{2}
 root :: Text -> TP Exp
 root c = do
@@ -693,17 +719,28 @@ xspace "\\mspace" =
        _                     -> mzero
 xspace "\\hspace" = do
   braces $ do
-    len <- many1 digit
-    scaleFactor <-
+    neg <- option 1 $ (-1) <$ char '-'
+    as <- option "" $ many1 digit
+    bs <- option "" $ char '.' *> many1 digit
+    let denominator = 10^(length bs)
+    as' <- if null as then pure 0 else stringToInteger as
+    bs' <- if null bs then pure 0 else stringToInteger bs
+    let numerator = (as' * denominator) + bs'
+    let n = neg * (numerator % denominator)
+    scaleFactor <- lexeme $
            1      <$ (string "em")
       <|> (1/12)  <$ (string "pt")
       <|> 6       <$ (string "in")
       <|> (50/21) <$ (string "cm")
-    case reads len of
-       ((n :: Integer,[]):_) -> return $ ESpace (fromIntegral n * scaleFactor)
-       _                     -> mzero
+    return $ ESpace (n * scaleFactor)
 xspace _ = mzero
 
+stringToInteger :: String -> TP Integer
+stringToInteger s =
+  case reads s of
+    ((n :: Integer, []):_) -> pure n
+    _ -> fail $ "Could not read " <> s <> " as Integer."
+
 mathop :: Text -> TP Exp
 mathop c =
   case c of
@@ -853,6 +890,7 @@ tSymbol sym =
          (\t -> EUnder False t acc) <$> texToken
        Just acc@(ESymbol TOver _) ->
          (\t -> EOver False t acc) <$> texToken
+       Just (ESymbol Alpha t) -> return $ EIdentifier t -- see #256
        Just x  -> return x
        Nothing
          | sym == "\\mod" -> do
@@ -967,6 +1005,7 @@ textCommands = M.fromList
   , ("\\v", option "v" $ try $ hacek <$> tok)
   , ("\\u", option "u" $ try $ breve <$> tok)
   , ("\\ ", return " ")
+  , ("\\quad", return "\x2000")
   ]
 
 parseC :: TP Text
