@@ -41,7 +41,7 @@ styleOps :: M.Map Text ([Exp] -> Exp)
 styleOps = M.fromList
           [ ("\\mathrm",     EStyled TextNormal)
           , ("\\mathup",     EStyled TextNormal)
-          , ("\\mathbf",     EStyled TextBold)
+          , ("\\mathbf",     EStyled TextNormal . (:[]) . EStyled TextBold)
           , ("\\boldsymbol", EStyled TextBold)
           , ("\\bm",         EStyled TextBold)
           , ("\\symbf",      EStyled TextBold)
@@ -154,7 +154,21 @@ symbolMapOverrides = M.fromList
   , ("\\Box",ESymbol Op "\9633")
   , ("\\Delta",EIdentifier "\916")
   , ("\\Diamond",ESymbol Op "\9671")
+  , ("\\Alpha",EIdentifier "\913")
+  , ("\\Beta",EIdentifier "\914")
   , ("\\Gamma",EIdentifier "\915")
+  , ("\\Epsilon",EIdentifier "\917")
+  , ("\\Zeta",EIdentifier "\918")
+  , ("\\Eta",EIdentifier "\919")
+  , ("\\Iota",EIdentifier "\921")
+  , ("\\Kappa",EIdentifier "\922")
+  , ("\\Mu",EIdentifier "\924")
+  , ("\\Nu",EIdentifier "\925")
+  , ("\\Omicron",EIdentifier "\927")
+  , ("\\omicron",EIdentifier "\959")
+  , ("\\Rho",EIdentifier "\929")
+  , ("\\Tau",EIdentifier "\932")
+  , ("\\Chi",EIdentifier "\935")
   , ("\\Im",ESymbol Ord "\8465")
   , ("\\Join",ESymbol Rel "\8904")
   , ("\\Lambda",EIdentifier "\923")
@@ -258,7 +272,6 @@ symbolMapOverrides = M.fromList
   , ("\\overbar",ESymbol Accent "\175")
   , ("\\overline",ESymbol TOver "\175")
   , ("\\overrightarrow",ESymbol Accent "\8407")
-  , ("\\perp",ESymbol Rel "\8869")
   , ("\\phi",EIdentifier "\981")
   , ("\\pi",EIdentifier "\960")
   , ("\\preceq",ESymbol Rel "\8828")
